<?php
/**
 * @name  Model_XXX
 * @description Classe héritant de "Model_esia" gèrent tout les accès à la base de donnée du plugin Server
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 08/11/2012
 */
include_once SNMP_CLASS;
include_once DEVICE_CLASS;
include_once MVCMODEL;

class Model_inventory extends Model_esia
{
    const PLUGIN_CRON_RULE = 'CRON_INVENTORY_RULES';
    const PLUGIN_CRON_IMPORT_SNMP = 'CRON_INVENTORY_IMPORT_SNMP';
    const PLUGIN_CRON_IMPORT_INTUNES = 'CRON_INVENTORY_IMPORT_INTUNES';
    const PLUGIN_CRON_IMPORT_POWERSHELL = 'CRON_INVENTORY_IMPORT_POWERSHELL';

    /**
	*** Selectionne une liste depuis la table inventory_component
	*** @param PDO $dbconnection un objet PDO
	*** @param string $typeInventory [HARDWARE || SOFTWARE]
	*** @return array liste de composant d'inventaire organisée par nom de device
	*** @throws \EsiaDbException
	**/
    public static function getListInventory($dbconnection,$typeInventory)
    {
        $allType = false;
        if($typeInventory == "ALL")
        {
             $allType = true;
        }
        $query = "SELECT inventory_is_built.built_id,g1.device_group_name,g1.device_group_id,
						esia_device.device_name,inventory_component.component_id,inventory_component.component_name,inventory_type.type_name,
						component_description,built_version,
						device_id,device_type
                FROM  esia_device
				JOIN inventory_is_built ON esia_device.device_id = inventory_is_built.built_device_id
				JOIN inventory_component ON inventory_component.component_id = inventory_is_built.built_hardware_id
				JOIN inventory_type ON inventory_type.type_id = inventory_component.component_type_id
				JOIN inventory_manufacturer ON inventory_manufacturer.manufacturer_id = inventory_component.component_manufacturer_id
                LEFT JOIN esia_device_group_in ON device_id = group_in_device_id
                LEFT JOIN esia_device_group as g1 ON device_group_id=group_in_device_group_id ";
        if(!$allType)
        {
            $query .= " WHERE inventory_type.type_type = :type ";
        }

		$query .= " ORDER BY device_name;";
        if(!$allType)
        {
            $params = array(':type' => $typeInventory);
        }
        $query = $dbconnection->prepare($query);
        if(!$allType)
        {
            $query = Model_esia::executeStatement($query,$params);
        }
        else
        {
            $query = Model_esia::executeStatement($query);
        }
        $nodetype = array();
        $arr = Model_esia::getWebPluginByNodeType($dbconnection);
        foreach($arr as $row)
        {
        	$nodetype[$row['type']] = $row['apache_path'].$row['control_page'];
        }
        $components = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			if(!array_key_exists($row->built_id, $components))
			{
				if(array_key_exists($row->device_type,$nodetype))
				{
					$page_control = Model_esia::replacePageControlVariable($nodetype[$row->device_type],$row);
				}
				else
				{
					$page_control = '/node_view.php?nodeid='.$row->device_id;
				}

				$components[$row->built_id] = array(
					'groupid'=> $row->device_group_id,
                    'group'=> $row->device_group_name,
                    'device_type'=>$row->device_type,
					'device' => $row->device_name,
                    'component_id' => $row->component_id,
					'component' => $row->component_name,
					'version' => $row->built_version,
					'description' => $row->component_description,
					'type' => $row->type_name,
					'page_control'=>$page_control
				);
			}
		}

        return $components;

    }

    public static function getComponentId($dbconnection,$componentName)
    {
        $query = "SELECT component_id FROM inventory_component WHERE component_name = :name ";
        $params = array(':name' => $componentName);
        $query = $dbconnection->prepare($query);
        $query = Model_esia::executeStatement($query,$params);

        $names = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			if(!array_key_exists($row->component_id, $names))
			{
				$names[$row->component_id] = array(
                    'component_id'=> $row->component_id
				);
			}
		}

        return $names;
    }

    /**
	*** Selectionne le nombre de constructeur qui sont installé
	*** @param PDO $dbconnection un objet PDO
	*** @return array tableau contenant le nom du constructeur et le nombre de composant installé
	*** @throws \EsiaDbException
	**/
    public static function getCountChassis($dbconnection, $filterDevices=NULL)
    {
        $sql = "SELECT inventory_manufacturer.manufacturer_name,
            COUNT(*) AS name_count
            FROM inventory_manufacturer
            JOIN inventory_component on inventory_manufacturer.manufacturer_id = inventory_component.component_manufacturer_id
            JOIN inventory_is_built ON inventory_component.component_id = inventory_is_built.built_hardware_id
            JOIN inventory_type on inventory_type.type_id = inventory_component.component_type_id
            WHERE manufacturer_name <> 'unknown'
            AND inventory_type.type_name = 'CHASSIS'";

        $params = array();
        if(isset($filterDevices))
        {
            $in_sql = Model_esia::pdo_array_to_SQL('built_device_id', $filterDevices, $params);
            $sql .= " AND inventory_is_built.built_device_id IN ($in_sql)";
        }

        $sql .= "
            GROUP BY manufacturer_name
            ORDER BY manufacturer_name
            LIMIT 10;
        ";

        $query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query, $params);

        $components = array();
        while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			if(!array_key_exists($row->manufacturer_name, $components))
			{
				$components[$row->manufacturer_name] = array(
                    'name'=> $row->manufacturer_name,
					'count' => $row->name_count
				);
			}
		}
        return $components;
    }

    public static function getListChassis($dbconnection, $filterDevices=NULL)
    {
        $sql = "SELECT component_name,component_description,COUNT(*) AS name_count
            FROM inventory_type
            INNER JOIN inventory_component on inventory_type.type_id = inventory_component.component_type_id
            INNER JOIN inventory_is_built ON inventory_component.component_id = inventory_is_built.built_hardware_id
			INNER JOIN esia_device ON esia_device.device_id = inventory_is_built.built_device_id
            WHERE inventory_type.type_name = 'CHASSIS'";

        $params = array();
        if(isset($filterDevices))
        {
            $in_sql = Model_esia::pdo_array_to_SQL('built_device_id', $filterDevices, $params);
            $sql .= " AND inventory_is_built.built_device_id IN ($in_sql)";
        }

        $sql .= "
            GROUP BY component_name,component_description
			ORDER BY name_count DESC
			LIMIT 10;
        ";

        $query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query, $params);

        $chassis = array();
        while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			if(!array_key_exists($row->component_name, $chassis))
			{
				$chassis[$row->component_name] = array(
                    'chassis' => $row->component_name,
                    'description' => $row->component_description,
                    'name_count'=> $row->name_count
				);
			}
		}
        return $chassis;
    }
    public static function getCountOS($dbconnection, $filterDevices=NULL)
    {
        $sql = "SELECT inventory_component.component_name,inventory_is_built.built_version,
            COUNT(*) as component_count
            FROM inventory_type
            JOIN inventory_component on inventory_type.type_id = inventory_component.component_type_id
			JOIN inventory_manufacturer ON inventory_manufacturer.manufacturer_id = inventory_component.component_manufacturer_id
            JOIN inventory_is_built ON inventory_component.component_id = inventory_is_built.built_hardware_id
            WHERE inventory_type.type_name = 'OS'
            AND manufacturer_name <> 'unknown'";

        $params = array();
        if(isset($filterDevices))
        {
            $in_sql = Model_esia::pdo_array_to_SQL('built_device_id', $filterDevices, $params);
            $sql .= " AND inventory_is_built.built_device_id IN ($in_sql)";
        }

        $sql .= "
            GROUP BY component_name,inventory_is_built.built_version
            ORDER BY component_name;
        ";

        $query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query, $params);

        $operatingSystems = array();
        while($row = $query->fetch(PDO::FETCH_OBJ))
		{
            $name =  $row->component_name.' '.$row->built_version;
			if(!array_key_exists($name, $operatingSystems))
			{
				$operatingSystems[$name] = array(
                    'name' =>$name,
                    'count' => $row->component_count
				);
			}
		}
        return $operatingSystems;
    }
    public static function getListOSbyVersion($dbconnection)
    {
        $query = "SELECT component_name,built_version,COUNT(*) AS name_count
            FROM inventory_type
            INNER JOIN inventory_component on inventory_type.type_id = inventory_component.component_type_id
            INNER JOIN inventory_is_built ON inventory_component.component_id = inventory_is_built.built_hardware_id
			INNER JOIN esia_device ON esia_device.device_id = inventory_is_built.built_device_id
            WHERE manufacturer_name <> 'unknown' AND inventory_type.type_name = 'OS'
			GROUP BY component_name,built_version
			ORDER BY name_count,component_name,built_version ASC
            LIMIT 10;";

        $query = $dbconnection->prepare($query);
        $query = Model_esia::executeStatement($query);

        $operatingSystems = array();
        while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			if(!array_key_exists($row->component_name, $operatingSystems))
			{
				$operatingSystems[$row->component_name] = array(
                    'software' => $row->component_name,
					'version' => $row->built_version,
                    'count'=>$row->name_count
				);
			}
		}
        return $operatingSystems;

    }
    /**
	*** Retourne une liste de logiciel
	*** @param PDO $dbconnection un objet PDO
	*** @return array Tableau contenant Le nom du device, le type de logiciel, son nom et sa version
	*** @throws \EsiaDbException
	**/
    public static function getListSoftwareByVersion($dbconnection, $filterDevices=NULL)
    {
        $sql = "SELECT  type_name, component_name,built_version,COUNT(*) AS name_count
            FROM public.inventory_type
            INNER JOIN inventory_component on inventory_type.type_id = inventory_component.component_type_id
            INNER JOIN inventory_is_built ON inventory_component.component_id = inventory_is_built.built_hardware_id
			INNER JOIN esia_device ON esia_device.device_id = inventory_is_built.built_device_id
            WHERE inventory_type.type_type = 'SOFTWARE'
			AND type_name NOT IN ('OS','UTILS','LIBRARY','OS-COMPONENT','APPLICATION-DATA','APPLICATION-MODULES','LANGUAGE')
			";

        $params = array();
        if(isset($filterDevices))
        {
            $in_sql = Model_esia::pdo_array_to_SQL('built_device_id', $filterDevices, $params);
            $sql .= " AND inventory_is_built.built_device_id IN ($in_sql)";
        }

        $sql .= "
            GROUP BY component_name,type_name,built_version
            ORDER by name_count DESC
            LIMIT 10;
        ";

        $query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query, $params);

        $softwares = array();
        while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			if(!array_key_exists($row->component_name, $softwares))
			{
				$softwares[$row->component_name] = array(

                    'software' => $row->component_name,
                    'type' => $row->type_name,
					'version' => $row->built_version,
                    'count' => $row->name_count
				);
			}
		}

        return $softwares;
    }
    public static function getListRules($dbconnection,$ruleId = NULL,$activate=FALSE)
    {
        $query = "SELECT rule_id,rule_name,rule_component_id,component_name,rule_version_target,rule_version_target_operator,
						rule_target,rule_match,rule_description,rule_activate
        FROM inventory_rules
		INNER JOIN inventory_component ON inventory_component.component_id = inventory_rules.rule_component_id";
        if(!empty($ruleId))
        {
            $query .= " WHERE rule_id = :rule_id";
        }
        if(!empty($activate))
        {
        	if(strpos($query,'#WHERE#') !==FALSE)
        	{
        		$query .= " AND rule_activate = TRUE";
        	}
        	else
        	{
        		$query .= " WHERE rule_activate = TRUE";
        	}
        }
        $query .= " ORDER BY rule_name";
        $query = $dbconnection->prepare($query);
        if(!empty($ruleId))
        {
            $params = array(':rule_id' => $ruleId);
            $query = Model_esia::executeStatement($query,$params);
        }
        else
        {
            $query = Model_esia::executeStatement($query);
        }
        $rules=array();
        while($row = $query->fetch(PDO::FETCH_OBJ))
        {
            if(!array_key_exists($row->rule_id,$rules))
            {
                $rules[$row->rule_id] = array(
                    'rule_id' => $row->rule_id,
                    'name' => $row->rule_name,
                    'component_id' => $row->rule_component_id,
                    'component_name' => $row->component_name,
                    'target_version' => $row->rule_version_target,
                    'operator_target' => $row->rule_version_target_operator,
                    'target' => json_decode($row->rule_target, TRUE),
                    'match' => json_decode($row->rule_match, TRUE),
                    'description' => $row->rule_description,
                    'activate' => $row->rule_activate,
                );
            }
        }
        return $rules;
    }

    /**
	*** Ajoute un constructeur
	*** @param PDO $dbconnection un objet PDO
	*** @param array|string $arrayManufacturer  tableau des informations du constructeur [name,oid,logo,pattern],
	*						si le paramètre est juste une chaine de caractère, on ajoute que le nom
	*** @return bool Retourne un insertion positive ou non
	*** @throws \EsiaDbException
	**/
    public static function getListAllConfig($dbconnection)
    {

    }

    public static function addManufacturer($dbconnection,$arrayManufacturer)
    {
    	$params = array();
    	$ret = NULL;

        $query = "INSERT INTO inventory_manufacturer(manufacturer_id, manufacturer_name) VALUES (DEFAULT,:name)
                    RETURNING manufacturer_id;";
        if(is_array($arrayManufacturer))
        {
        	$query = "INSERT INTO inventory_manufacturer (manufacturer_id,manufacturer_name, manufacturer_oid,
						manufacturer_logo_path, manufacturer_pattern) VALUES
           				(DEFAULT,:name,:oid,:logo,:pattern) RETURNING manufacturer_id;";
        	$params = array(':name' => $arrayManufacturer['name'],':oid' => $arrayManufacturer['oid'],
        					':logo' => $arrayManufacturer['logo'],':pattern' => $arrayManufacturer['pattern']);
        }
        else
        {
        	$params = array(':name' => $arrayManufacturer);
        }
        $query = $dbconnection->prepare($query);
        $query = Model_esia::executeStatement($query,$params);
        $ret=$query->fetch(PDO::FETCH_OBJ);

        return $ret->manufacturer_id;
    }
    public static function getGroup($dbconnection,$idGroup)
    {
        $group = array();

        $query = "SELECT device_group_id, device_group_name FROM esia_device_group WHERE device_group_id = :id ;";
        $query = $dbconnection->prepare($query);
        $params = array(':id' => $idGroup);
        $query = Model_esia::executeStatement($query,$params);

        while($row = $query->fetch(PDO::FETCH_OBJ))
        {
            if(!array_key_exists($row->device_group_id,$group))
            {
                $group[$row->device_group_id] = array(
                  'id' => $row->device_group_id,
                  'name' => $row->device_group_name
                );
            }
        }

        return $group;
    }
    public static function getAllDevice($dbconnection, $idDevice="ALL")
    {
        $sql = "SELECT device_id, device_name FROM esia_device";

        $params = array();

        // If not an array or numeric id get ALL devices
        if(is_array($idDevice))
        {
            $in_sql = Model_esia::pdo_array_to_SQL('device_id', $idDevice, $params);
            $sql .= " WHERE device_id IN ($in_sql)";
        }
        elseif(is_numeric($idDevice))
        {
            $sql .= " WHERE device_id = :id";
            $params = array(':id' => $idDevice);
        }
        $sql .= " ORDER BY device_name;";

        $query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query, $params);

        $devices = array();
        while($row = $query->fetch(PDO::FETCH_OBJ))
        {
            $devices[$row->device_id] = array(
                'id' => $row->device_id,
                'name' => $row->device_name,
            );
        }

        return $devices;
    }

    /**
	*** Ajoute un constructeur
	*** @param PDO $dbconnection un objet PDO
	*** @param enteger Identifiant de device ou du groupe
	*** @param string Différenciation entre les device et les groupes (L'action Groupe se fait avec "GROUP")
	*** @return bool Retourne un insertion positive ou non
	*** @throws \EsiaDbException
	**/
    public static function getDeviceComponent($dbconnection,$ids,$groupOrDevice = "",$component="",$type=null)
    {
        $data = self::getDeviceComponentsList($dbconnection,$ids,$groupOrDevice,$component,$type);
        $ret = array();
        foreach($data as $device_id => $list)
        {
            $ret[$device_id] = array_shift($list);
        }
        return $ret;
    }

    public static function getDeviceComponentsList($dbconnection,$ids,$groupOrDevice = "",$component="",$type=null)
    {
        $query = "SELECT inventory_is_built.built_id,g1.device_group_name,g1.device_group_id,
						esia_device.device_name,inventory_component.component_name,inventory_type.type_name,
						component_description,built_version,built_script_insert,
						device_id,device_type,built_macaddress,built_serial_number,built_ipaddress
                FROM  esia_device
				JOIN inventory_is_built ON esia_device.device_id = inventory_is_built.built_device_id
				JOIN inventory_component ON inventory_component.component_id = inventory_is_built.built_hardware_id
				JOIN inventory_type ON inventory_type.type_id = inventory_component.component_type_id
				JOIN inventory_manufacturer ON inventory_manufacturer.manufacturer_id = inventory_component.component_manufacturer_id
                LEFT JOIN esia_device_group_in ON device_id = group_in_device_id
                LEFT JOIN esia_device_group as g1 ON device_group_id=group_in_device_group_id
                WHERE ";
        $param = array();
        if(!empty($component))
        {
            if(is_numeric($component))
            {
                $query = $query ." component_id = :compo AND ";
            }
            else
            {
                $query = $query ." component_name = :compo AND ";
            }
            $param[':compo'] = $component;
        }
        if($groupOrDevice == "GROUP")
        {
            $query = $query ." device_group_id IN (SELECT group_id FROM ";
            if(is_array($ids))
            {
                foreach($ids as $id)
                {
                    if(!is_numeric($id))
                    {
                        throw new EsiaDbException('$group array must be numeric');
                    }
                }
                $arrids = implode(',', $ids);
                $query = $query."getallchilddevicegroups('{ $arrids }')";
            }
            else
            {
                $param[':group'] = $ids;
                $query = $query."getallchilddevicegroup(:group)";
            }
            $query = $query.")";
        }
        else
        {
            if(is_array($ids))
            {
                $inarr = Model_esia::pdo_array_to_SQL('nodeid', $ids, $param);
                $query = $query ." esia_device.device_id IN($inarr)";
            }
            else
            {
                $query .=" esia_device.device_id = :id";
                $param[':id'] = $ids;
            }
        }
        if(!empty($type))
        {
        	$query = $query ." AND built_script_insert = :script";
        	$param[':script'] = $type;
        }

        $query = $dbconnection->prepare($query);
        $query = Model_esia::executeStatement($query,$param);
        $device=array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			if(!array_key_exists($row->device_id, $device))
			{
                $device[$row->device_id] = array();
			}
            $device[$row->device_id][] = array(
                'built_id'=>$row->built_id,
                'groupid'=> $row->device_group_id,
                'group'=> $row->device_group_name,
                'device_type'=>$row->device_type,
                'device' => $row->device_name,
                'component' => $row->component_name,
                'version' => $row->built_version,
                'description' => $row->component_description,
                'type' => $row->type_name,
                'serial'=>$row->built_serial_number,
                'macadress'=>$row->built_macaddress,
                'ip'=>$row->built_ipaddress,
                'script'=> $row->built_script_insert
            );
		}
        return $device;

    }
    public static function getAllGroupDevice($dbconnection)
    {
        $group = array();

        $query = "SELECT device_group_id,device_group_name FROM esia_device_group";
        $query = $dbconnection->prepare($query);
        $query = Model_esia::executeStatement($query);

        while($row = $query->fetch(PDO::FETCH_OBJ))
        {
            if(!array_key_exists($row->device_group_id,$group))
            {
                $group[$row->device_group_id] = array(
                    'id' => $row->device_group_id,
                    'name' => $row->device_group_name
                );
            }
        }

        return $group;
    }

     /**
	*** Ajoute un type de matériel ou logiciel
	*** @param PDO $dbconnection un objet PDO
	*** @param string $name nom du type
	*** @param string $type type
	*** @return integer Retourne l'id du type inséré
	*** @throws \EsiaDbException
	**/
    public static function addType($dbconnection,$name,$type)
    {
        $query = "INSERT INTO inventory_type (type_id,type_name,type_type) VALUES (DEFAULT,:name,:type) RETURNING type_id;";
        $params = array(':name' => $name,':type' => $type);
        $query = $dbconnection->prepare($query);
        $query = Model_esia::executeStatement($query,$params);
        $ret=$query->fetch(PDO::FETCH_OBJ);

        return $ret->type_id;
    }
    /**
	*** Ajoute un componement
	*** @param PDO $dbconnection un objet PDO
	*** @param array $arrayComponent tableau des informations du component
	*** @return bool Retourne un insertion positive ou non
	*** @throws \EsiaDbException
	**/
    public static function addComponent($dbconnection,$name,$description,$model,$type,$manufacturer,$other=NULL)
    {
        try
        {
        	$params = array(':name' =>$name,
        		':description' => $description,
        		':model' => $model,
        		':type_id' => $type,
        		':manufacturer_id' => $manufacturer);

            $query = "INSERT INTO inventory_component (component_id,component_name,component_description,
						component_model,component_type_id,component_manufacturer_id,component_json)
                VALUES (DEFAULT,:name,:description,:model,";
            if(is_numeric($type))
            {
                $query .= ":type_id,";
            }
            else
            {
                $query .= "(SELECT type_id FROM inventory_type WHERE type_name = :type_id),";
            }
            if(is_numeric($manufacturer))
            {
                $query .= ":manufacturer_id";
            }
            else
            {
                $query .= "(SELECT manufacturer_id FROM inventory_manufacturer WHERE manufacturer_name = :manufacturer_id)";
            }
            if(!empty($other))
            {
            	$query .= ",:json";
            	$params[':json']=$other;
            }
            else
            {
            	$query .=",NULL";
            }
            $query .= ") RETURNING component_id;";
            $query = $dbconnection->prepare($query);
            $query = Model_esia::executeStatement($query,$params);
            $ret=$query->fetch(PDO::FETCH_OBJ);
        }
        catch(EsiaDbException $err)
        {
           throw new EsiaDbException($err->getMessage());
        }
        return $ret->component_id;
    }
    /**
     *** Ajoute un composant logicielsur le noeud
     *** @param PDO $dbconnection un objet PDO
     *** @param array $arrayComponent tableau des informations du component
     *** @return bool Retourne un insertion positive ou non
     *** @throws \EsiaDbException
     **/
    public static function addSoftwareComponentOnNode($dbconnection,$node,$name,$description,$component_id,$version,$other=NULL,$scanner='SNMP')
    {
    	try
    	{
    		$params = array(':nodeid'=>$node,':checkid'=>$node,
    			':name' =>$name,
    			':description' => $description,
    			':checkname'=>$name,
    			':component_id' => $component_id,
    			':version' => $version);
    		$query ="INSERT INTO inventory_is_built(
								built_name,
								built_description,
								built_device_id,
								built_hardware_id,
								buil_installed_date,
								built_version,
								built_json,
								built_script_insert)
					SELECT :name,:description,:nodeid,:component_id,NOW(),:version";
    		if(!empty($other))
    		{
    			$query .= ",:json";
    			$params[':json']=$other;
    		}
    		else
    		{
    			$query .=",NULL";
    		}
    		if(!empty($scanner))
    		{
    			$query .= ",:scanner";
    			$params[':scanner']=$scanner;
    		}
    		else
    		{
    			$query .=",NULL";
    		}
    		$query.=" WHERE NOT EXISTS (SELECT built_id
									FROM inventory_is_built
									WHERE built_device_id=:checkid AND built_name=:checkname)
				  RETURNING built_id";

    		$query = $dbconnection->prepare($query);
    		$query = Model_esia::executeStatement($query,$params);
    		$ret=$query->fetch(PDO::FETCH_OBJ);
    	}
    	catch(EsiaDbException $err)
    	{
    		throw new EsiaDbException($err->getMessage());
    	}
    	if(!empty($ret))
    	{
    		return $ret->built_id;
    	}
    	return null;
    }
    /**
     *** mets à jour un composant logiciel sur le noeud
     *** @param PDO $dbconnection un objet PDO
     *** @param array $arrayComponent tableau des informations du component
     *** @return bool Retourne un insertion positive ou non
     *** @throws \EsiaDbException
     **/
    public static function updateSoftwareComponentOnNode($dbconnection,$builtid,$description,$version,$other=NULL)
    {
    	try
    	{
    		$params = array(':built_id'=>$builtid,
    			':description' => $description);

    		$query ="UPDATE inventory_is_built SET built_last_update=NOW(),built_description=:description";
    		if(!empty($other))
    		{
    			$query .= ",built_json = :json";
    			$params[':json']=$other;
    		}
    		if(!empty($version))
    		{
    			$query .= ",built_version = :version";
    			$params[':version']=$version;
    		}
    		$query.=" WHERE built_id = :built_id;";

    		$query = $dbconnection->prepare($query);
    		$query = Model_esia::executeStatement($query,$params);
    	}
    	catch(EsiaDbException $err)
    	{
    		throw new EsiaDbException($err->getMessage());
    	}
    	return $builtid;
    }
    /**
     *** Ajoute un composant matériel sur le noeud
     *** @param PDO $dbconnection un objet PDO
     *** @param array $arrayComponent tableau des informations du component
     *** @return bool Retourne un insertion positive ou non
     *** @throws \EsiaDbException
     **/                 //  addHardwareComponentOnNode($dbconnection,$node->getId(),$line['model'],$descr,$line['serial'],$componentID,$line['firmwareVersion'],NULL);
    public static function addHardwareComponentOnNode($dbconnection,$node,$name,$description,$serial,$component_id,$version,$other=NULL,$script='SNMP')
    {
    	$ret=NULL;
    	try
    	{
    		$params = array(':nodeid'=>$node,
    			':name' =>$name,
    			':description' => $description,
    			':serial' => $serial,
    			':component_id' => $component_id,
    			':version' => $version,
    			':script'=>$script
    		);
    		$query ="INSERT INTO inventory_is_built(
								built_name,
								built_description,
								built_serial_number,
								built_device_id,
								built_script_insert,
								built_hardware_id,
								buil_installed_date,
								built_version,
								built_json)
					SELECT :name,:description,:serial,:nodeid,:script,:component_id,NOW(),:version";
    		if(!empty($other))
    		{
    			$query .= ",:json";
    			$params[':json']=$other;
    		}
    		else
    		{
    			$query .=",NULL";
    		}
    		if(!empty($serial))
    		{
    			$query.=" WHERE NOT EXISTS (SELECT built_id
									FROM inventory_is_built
									WHERE built_device_id=:checkid  AND built_serial_number=:checkserial)";
    			$params[':checkserial']=$serial;
    			$params[':checkid']=$node;
    		}
    		else
    		{
    			$query.=" WHERE NOT EXISTS (SELECT built_id
									FROM inventory_is_built
									WHERE built_name=:checkname AND built_device_id=:checkid AND built_hardware_id=:checkcomponent )";
    			$params[':checkname']=$name;
    			$params[':checkid']=$node;
    			$params[':checkcomponent']=$component_id;
    		}
            $query .= "  RETURNING built_id";
    		$query = $dbconnection->prepare($query);
    		$query = Model_esia::executeStatement($query,$params);
    		$ret=$query->fetch(PDO::FETCH_OBJ);
    	}
    	catch(EsiaDbException $err)
    	{
    		throw new EsiaDbException($err->getMessage());
    	}
    	if(!empty($ret))
    	{
    		return $ret->built_id;
    	}
    	return NULL;
    }
    /**
     *** Ajoute un composant matériel de type 'NETINT' sur le noeud
     *** @param PDO $dbconnection un objet PDO
     *** @param array $arrayComponent tableau des informations du component
     *** @return bool Retourne un insertion positive ou non
     *** @throws \EsiaDbException
     **/
    public static function addNetinComponentOnNode($dbconnection,$node,$name,$description,$serial,$component_id,$mac,$ip,$other=NULL,$script='SNMP')
    {
    	try
    	{
    		$params = array(':nodeid'=>$node,
    			':name' =>$name,
    			':description' => $description,
    			':serial' => $serial,
    			':component_id' => $component_id,
    			':script'=>$script
    		);

    		$query ="INSERT INTO inventory_is_built(
								built_name,
								built_description,
								built_serial_number,
								built_device_id,
								built_script_insert,
								built_hardware_id,
								buil_installed_date,
								built_macaddress,
								built_ipaddress,
								built_json)
					SELECT :name,:description,:serial,:nodeid,:script,:component_id,NOW(),:version";
    		if(!empty($mac))
    		{
    			$query .= ",:mac";
    			$params[':mac']=$mac;
    		}
    		else
    		{
    			$query .=",NULL";
    		}
    		if(!empty($other))
    		{
    			$query .= ",:ip";
    			$params[':ip']=$ip;
    		}
    		else
    		{
    			$query .=",NULL";
    		}
    		if(!empty($other))
    		{
    			$query .= ",:json";
    			$params[':json']=$other;
    		}
    		else
    		{
    			$query .=",NULL";
    		}
    		if(!empty($serial))
    		{
    			$query.=" WHERE NOT EXISTS (SELECT built_id
									FROM inventory_is_built
									WHERE built_device_id=:checkid  AND built_serial_number=:checkserial)";
    			$params[':checkserial']=$serial;
    			$params[':checkid']=$node;
    		}
    		else
    		{
    			$query.=" WHERE NOT EXISTS (SELECT built_id
									FROM inventory_is_built
									WHERE built_name=:checkname AND built_device_id=:checkid AND built_hardware_id=:checkcomponent )";
    			$params[':checkname']=$name;
    			$params[':checkid']=$node;
    			$params[':checkcomponent']=$component_id;
    		}
    		$query .= "  RETURNING built_id";
    		$query = $dbconnection->prepare($query);
    		$query = Model_esia::executeStatement($query,$params);
    		$ret=$query->fetch(PDO::FETCH_OBJ);
    	}
    	catch(EsiaDbException $err)
    	{
    		throw new EsiaDbException($err->getMessage());
    	}
    	if(!empty($ret))
    	{
    		return $ret->built_id;
    	}
    }
    /**
     *** Met à jour un composant netint sur le noeud
     *** @param PDO $dbconnection un objet PDO
     *** @param array $arrayComponent tableau des informations du component
     *** @return bool Retourne un insertion positive ou non
     *** @throws \EsiaDbException
     **/
    public static function updateNetinComponentOnNode($dbconnection,$builtid,$description,$version,$mac,$ip,$other=NULL)
    {
    	try
    	{
    		$params = array(':built_id'=>$builtid,
    			':description' => $description);

    		$query ="UPDATE inventory_is_built SET built_last_update=NOW(),built_description=:description";
    		if(!empty($other))
    		{
    			$query .= ",built_json = :json";
    			$params[':json']=$other;
    		}
    		if(!empty($version))
    		{
    			$query .= ",built_version = :version";
    			$params[':version']=$version;
    		}
    		if(!empty($mac))
    		{
    			$query .= ",built_macaddress = :mac";
    			$params[':mac']=$mac;
    		}
    		if(!empty($ip))
    		{
    			$query .= ",built_ipaddress = :ip";
    			$params[':ip']=$ip;
    		}
    		$query.=" WHERE built_id = :built_id;";

    		$query = $dbconnection->prepare($query);
    		$query = Model_esia::executeStatement($query,$params);
    	}
    	catch(EsiaDbException $err)
    	{
    		throw new EsiaDbException($err->getMessage());
    	}
    	return $builtid;
    }
    /**
     *** Met à jour un composant netint sur le noeud
     *** @param PDO $dbconnection un objet PDO
     *** @param array $arrayComponent tableau des informations du component
     *** @return bool Retourne un insertion positive ou non
     *** @throws \EsiaDbException
     **/
    public static function updateHardwareComponentOnNode($dbconnection,$builtid,$description,$version,$other)
    {
    	try
    	{
    		$params = array(':built_id'=>$builtid,
    			':description' => $description);

    		$query ="UPDATE inventory_is_built SET built_last_update=NOW(),built_description=:description";
    		if(!empty($other))
    		{
    			$query .= ",built_json = :json";
                if( is_array($other))
                {
                    $other = json_encode($other);
                }
    			$params[':json']=$other;
    		}
    		if(!empty($version))
    		{
    			$query .= ",built_version = :version";
    			$params[':version']=$version;
    		}
    		$query.=" WHERE built_id = :built_id;";

    		$query = $dbconnection->prepare($query);
    		$query = Model_esia::executeStatement($query,$params);
    	}
    	catch(EsiaDbException $err)
    	{
    		throw new EsiaDbException($err->getMessage());
    	}
    	return $builtid;
    }
    /**
     *** Met à jour un composant netint sur le noeud
     *** @param PDO $dbconnection un objet PDO
     *** @param array $arrayComponent tableau des informations du component
     *** @return bool Retourne un insertion positive ou non
     *** @throws \EsiaDbException
     **/
    public static function deleteComponentOnNode($dbconnection,$builtid,$nodeid)
    {
    	$params = array(':built_id'=>$builtid,
    		':node' => $nodeid);

    	$query ="DELETE FROM inventory_is_built  WHERE built_id = :built_id AND built_device_id=:node;";
    	$query = $dbconnection->prepare($query);
    	$query = Model_esia::executeStatement($query,$params);
    }
    public static function addBuilt($dbconnection,$arrayBuild)
    {
        $query = "INSERT INTO public.inventory_is_built(
            built_id, built_name, built_description, built_serial_number, built_device_id, built_hardware_id, built_macaddress, built_ipaddress, buil_installed_date, built_json)
            VALUES (DEFAULT, :name, :description, :serial_number, :device_id, :hardware_id, :macaddress, :ipaddress, :installed_date, :json);";
        if(is_numeric($arrayBuild['device_id']))
        {
            $query .= ":device_id,";
        }
        else
        {
            $query .= "(SELECT device_id FROM device WHERE device_name = :device_id),";
        }
        if(is_numeric($arrayBuild['hardware_id']))
        {
            $query .= ":hardware_id,";
        }
        else
        {
            $query .= "(SELECT component_id FROM inventory_component WHERE component_name = :hardware_id),";
        }
        $params = array(':name' => $arrayBuild['name'],':description' => $arrayBuild['description'],':serial_number' => $arrayBuild['serial_number'],
                        ':device_id' => $arrayBuild['device_id'],':hardware_id' => $arrayBuild['hardware_id'],
                        ':macaddress' => $arrayBuild['macaddress'],':ipaddress' => $arrayBuild['ipaddress'],
                        ':installed_date' => $arrayBuild['installed_date'],':json' => $arrayBuild['json']);
        $query = $dbconnection->prepare($query);
        $query = Model_esia::executeStatement($query,$params);
    }
     /**
	*** Ajoute une règle
	*** @param PDO $dbconnection un objet PDO
	*** @param array $arrayRules tableau des informations de la regle (rule_target et rule_match doivent préciser groups[ids] et/ou devices[ids])
	*** @return bool la vleur de l'insertion
	*** @throws \EsiaDbException
	**/
    public static function addRules($dbconnection,$arrayRules)
    {
        $sql = "INSERT INTO inventory_rules(
            rule_id, rule_name, rule_component_id, rule_version_target, rule_version_target_operator, rule_target, rule_match, rule_description, rule_activate)
            VALUES (DEFAULT, :name, ";

        if(is_numeric($arrayRules['component_id']))
        {
            $sql .= ":component_id,";
        }
        else
        {
            $sql .= "(SELECT component_id FROM inventory_component WHERE component_name = :component_id),";
        }
        $sql .= " :version, :operator, :target, :match, :description, :activate) RETURNING rule_id;";

        $params = array(
            ':name' => $arrayRules['name'],
            ':component_id' => $arrayRules['component_id'],
            ':version' => $arrayRules['target_version'],
            ':operator' => $arrayRules['operator_target'],
            ':target' => empty($arrayRules['target']) ? NULL : json_encode($arrayRules['target']),
            ':match' => empty($arrayRules['match']) ? NULL : json_encode($arrayRules['match']),
            ':description' => $arrayRules['description'],
            ':activate' => empty($arrayRules['activate']) ? 0 : 1,
        );

        $query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query, $params);

        return $query->fetch(PDO::FETCH_COLUMN);
    }
    public static function deleteRule($dbconnection,$idRule)
    {
    	$params = array(':idRule' => $idRule);
    	$query = "UPDATE inventory_is_built SET built_rule_match_worst_status=NULL, built_rule_match_worst_rule_id=NULL WHERE built_rule_match_worst_rule_id = :idRule";
    	$query = $dbconnection->prepare($query);
    	$query = Model_esia::executeStatement($query,$params);
        $query = "DELETE FROM inventory_rules WHERE rule_id = :idRule";
        $query = $dbconnection->prepare($query);
        $query = Model_esia::executeStatement($query,$params);
    }
    /**
	*** Update an inventory rule
	*** @param PDO $dbconnection PDO object
    *** @param integer $ruleId id of the rule to update
	*** @param array $ruleData data to set in table inventory_rule for its $ruleId
	*** @throws \EsiaDbException
	**/
    public static function updateRule($dbconnection, $ruleId, $ruleData)
    {
        if(!isset($ruleId))
		{
			throw new EsiaDbException("Missing rule id");
		}

        $sql = "
            UPDATE inventory_rules
            SET rule_name=:name, rule_component_id=:component_id, rule_version_target=:version, rule_version_target_operator=:operator, rule_target=:target, rule_match=:match, rule_description=:description, rule_activate=:activate
            WHERE rule_id=:id
        ";

        $params = array(
            ':id' => $ruleId,
            ':name' => $ruleData['name'],
            ':component_id' => $ruleData['component_id'],
            ':version' => $ruleData['target_version'],
            ':operator' => $ruleData['operator_target'],
            ':target' => empty($ruleData['target']) ? NULL : json_encode($ruleData['target']),
            ':match' => empty($ruleData['match']) ? NULL : json_encode($ruleData['match']),
            ':description' => $ruleData['description'],
            ':activate' => empty($ruleData['activate']) ? 0 : 1,
        );

        $query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query, $params);
    }
    public static function updateMatchToRule($dbconnection,$match,$idRule)
    {

        $update = "UPDATE inventory_is_built
                        SET built_rule_match_worst_rule_id = :rule_id,
                            built_rule_match_worst_status  = :state
	                WHERE built_id= :id ;";
        $query = $dbconnection->prepare($update);
        foreach($match as $key => $state)
        {
            $params = array(':id'=>$key,':rule_id'=>$idRule,':state'=>$state);
            $query = Model_esia::executeStatement($query,$params);
        }
        return $query;
    }
    private static function formatDataCronTask($nextExec, $interval=0, $parameters=array())
    {
        if(!is_numeric($nextExec))
        {
            throw new EsiaDbException('Wrong next exec parameter must be a timestamp');
        }

        // Cron parameters
        $pluginParams = '';
        if(!empty($parameters['g']) && is_array($parameters['g']))
        {
            $pluginParams = '-g ' . implode(',', $parameters['g']);
        }
        unset($parameters['g']);
        foreach($parameters as $key => $param)
        {
            $pluginParams .= " -$key $param";
        }

        // Interval
        switch($interval)
        {
            case 'day':
                $interval = '1 day';
                break;
            case 'week':
                $interval = '1 week';
                break;
            case 'month':
                $interval = '1 month';
                break;
            case 'year':
                $interval = '1 year';
                break;
            default:
                $interval = 0;
                break;
        }

        return array(
            'parameters' => $pluginParams,
            'interval' => $interval,
            'timeexec' => $nextExec,
            'logfile' => 'inventory_$SERVICEID_$DATE.log',
            'retrie' => 1,
            'retrie_interval' => 15,
        );
    }
	public static function addScan($dbconnection, $typeNode, $type='', $nextExec, $interval=0, $parameters=array())
    {
        // Get plugin cron by type
        $pluginName = '';
        $typeNode = strtolower($typeNode);
        if($typeNode == 'rule')
        {
            $pluginName = self::PLUGIN_CRON_RULE;
        }
        elseif($typeNode == 'import')
        {
            $type = strtolower($type);
            if($type == 'snmp')
            {
                $pluginName = self::PLUGIN_CRON_IMPORT_SNMP;
            }
            elseif($type == 'intunes')
            {
                $pluginName = self::PLUGIN_CRON_IMPORT_INTUNES;
            }
            elseif($type == 'powershell')
            {
                $pluginName = self::PLUGIN_CRON_IMPORT_POWERSHELL;
            }
        }
        if(empty($pluginName))
        {
            throw new EsiaDbException('Wrong cron plugin type');
        }

        $plugin = Model_esia::getCronPlugin($dbconnection, $pluginName)[0];

        $cronData = self::formatDataCronTask($nextExec, $interval, $parameters);

        $id = Model_esia::addCronTask($dbconnection, $plugin['id'], $cronData['parameters'], $cronData['interval'], $cronData['timeexec'], $cronData['logfile'], $cronData['retrie'], $cronData['retrie_interval']);

        return $id;
    }
    public static function updateScan($dbconnection, $taskId, $nextExec, $interval=0, $parameters=array())
    {
        if(empty($taskId) || !is_numeric($taskId))
        {
            throw new EsiaDbException('Missing task ID');
        }

        $cronData = self::formatDataCronTask($nextExec, $interval, $parameters);

        Model_esia::updateCronTask($dbconnection, $taskId, $cronData['parameters'], $cronData['interval'], $cronData['timeexec'], $cronData['logfile'], $cronData['retrie'], $cronData['retrie_interval']);
    }
    // Requete Module Rapport
    /**
    *** Selectionne le nombre de constructeur qui sont installé
    *** @param PDO $dbconnection un objet PDO
    *** @param integer $userId identifiant de l'utilisateur qui a le droit sur les noeuds
    *** @param integer|array $nodeIds (optionnel) identifiants des noeuds a filtrer
    *** @param integer|array $groupIds (optionnel) identifiants des groupe de noeuds a filtrer
    *** @param string $type_search (optionnel) si fourni, filtre sur le type
    *** @param integer $limit (optionnel) limite le nombre de résultats
    *** @return array tableau contenant le nom du constructeur et le nombre de composant installé
    *** @throws \EsiaDbException
    **/
    public static function getCountManufacturer($dbconnection, $userId, $nodeIds=NULL, $groupIds=NULL, $type_search=NULL, $limit=NULL)
    {
        $sql = "SELECT inventory_manufacturer.manufacturer_name, COUNT(inventory_is_built.built_name) AS name_count
            FROM inventory_manufacturer
            INNER JOIN inventory_component ON inventory_manufacturer.manufacturer_id = inventory_component.component_manufacturer_id
            INNER JOIN inventory_is_built ON inventory_component.component_id = inventory_is_built.built_hardware_id
            WHERE manufacturer_name <> 'unknown'
            AND built_device_id IN (SELECT * FROM getUserAccessNode(:user_id))";

        $params = array(':user_id' => $userId);

        // Filter on nodes
        if(!empty($nodeIds))
        {
            if(is_numeric($nodeIds))
            {
                $sql .= " AND built_device_id = :device_id";
                $params[':device_id'] = $nodeIds;
            }
            elseif(is_array($nodeIds))
            {
                $nodeIds = array_filter($nodeIds, 'is_numeric');
                if(!empty($nodeIds))
                {
                    $in_sql = Model_esia::pdo_array_to_SQL('device_id', $nodeIds, $params);
                    $sql .= " AND built_device_id IN ($in_sql)";
                }
            }
        }

        // Filter on groups
        if(!empty($groupIds))
        {
            $sqlGroup = '';
            if(is_numeric($groupIds))
            {
                $sqlGroup = "SELECT group_id FROM getallchilddevicegroup(:group_id)";
                $param[':group_id'] = $groupIds;
            }
            elseif(is_array($groupIds))
            {
                $groupIds = array_filter($groupIds, 'is_numeric');
                if(!empty($groupIds))
                {
                    $arrids = implode(',', $groupIds);
                    $sqlGroup = "SELECT group_id FROM getallchilddevicegroups('{ $arrids }')";
                }
            }

            if(!empty($sqlGroup))
            {
                // No join on group_in_device_id to prevent multiples results if a node is in more than one group
                $sql .= " AND built_device_id IN (
                    SELECT group_in_device_id FROM esia_device_group_in WHERE group_in_device_group_id IN ($sqlGroup)
                )";
            }
        }

        // Filter on type
        if(is_string($type_search))
        {
            switch($type_search)
            {
                case 'OS':
                    $sqlWhereType = " WHERE type_type = 'SOFTWARE' AND type_name = 'OS'";
                    break;
                case 'CHASSIS':
                    $sqlWhereType = " WHERE type_type = 'HARDWARE' AND type_name = 'CHASSIS'";
                    break;
                case 'HARDWARE':
                    $sqlWhereType = " WHERE type_type = 'HARDWARE' AND type_name <> 'CHASSIS'";
                    break;
                case 'SOFTWARE':
                    $sqlWhereType = " WHERE type_type = 'SOFTWARE' AND type_name NOT IN ('OS', 'UTILS', 'LIBRARY', 'OS-COMPONENT', 'APPLICATION-DATA', 'APPLICATION-MODULES', 'LANGUAGE')";
                    break;
                case 'ALL_HARDWARE':
                    $sqlWhereType = " WHERE type_type = 'HARDWARE'";
                    break;
                case 'ALL_SOFTWARE':
                    $sqlWhereType = " WHERE type_type = 'SOFTWARE'";
                    break;
                default:
                    // If type_search == ALL or is not in the list there is no filter
                    $sqlWhereType = "";
                    break;
            }
            if(!empty($sqlWhereType))
            {
                $sql .= " AND component_type_id IN (SELECT type_id FROM inventory_type $sqlWhereType)";
            }
        }

        $sql .= " GROUP BY inventory_manufacturer.manufacturer_name ORDER BY name_count DESC";
        if(!empty($limit) && is_numeric($limit))
        {
            $sql .= " LIMIT :limit";
            $params[':limit'] = $limit;
        }

        $query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query, $params);

        $manufacturers = array();
        while($row = $query->fetch(PDO::FETCH_OBJ))
        {
            if(!isset($manufacturers[$row->manufacturer_name]))
            {
                $manufacturers[$row->manufacturer_name] = array(
                    'name' => $row->manufacturer_name,
                    'count' => $row->name_count,
                );
            }
        }
        return $manufacturers;
    }

    /**
     *** Retourne un tableau associatif id => nom du fabricant
     *** @param PDO $dbconnection un objet PDO
     *** @param string|integer $id optionnel nom au id du fabricant.
     *** @return array un tableau associatif id => nom du fabricant
     *** @throws \EsiaDbException
     **/
    public static function getImportManufacturer($dbconnection,$id=NULL)
    {
    	$query = "SELECT manufacturer_id, manufacturer_name
    			  FROM inventory_manufacturer";
    	$params = array();
    	if(!empty($id))
    	{
    		if(is_numeric($id))
    		{
    			$query .= " WHERE manufacturer_id=?";
    		}
    		else
    		{
    			$query .= " WHERE manufacturer_name=?";
    		}
    		$params[]=$id;
    	}
    	$query = $dbconnection->prepare($query);
    	$query = Model_esia::executeStatement($query,$params);
    	$arr = array();
    	while($row = $query->fetch(PDO::FETCH_OBJ))
    	{
    		$arr[$row->manufacturer_id]=$row->manufacturer_name;
    	}
    	return $arr;
    }
    /**
     *** Retourne un tableau associatif id => type de matériel
     *** @param PDO $dbconnection un objet PDO
     *** @param string|integer $id optionnel nom au id du type.
     *** @return array un tableau associatif id => nom du type
     *** @throws \EsiaDbException
     **/
    public static function getImportType($dbconnection,$id=NULL)
    {
    	$query = "SELECT type_id, type_name
    			  FROM inventory_type";
    	$params = array();
    	if(!empty($id))
    	{
    		if(is_numeric($id))
    		{
    			$query .= " WHERE type_id=?";
    		}
    		else
    		{
    			$query .= " WHERE type_name=?";
    		}
    		$params[]=$id;
    	}
    	$query = $dbconnection->prepare($query);
    	$query = Model_esia::executeStatement($query,$params);
    	$arr = array();
    	while($row = $query->fetch(PDO::FETCH_OBJ))
    	{
    		$arr[$row->type_id]=$row->type_name;
    	}
    	return $arr;
    }
    /**
     *** Retourne un tableau associatif d'un composant array('model'=>,'manufacturer'=>,'type'=> );
     *** @param PDO $dbconnection un objet PDO
     *** @param string|integer $id optionnel modèle ou id du composant.
     *** @return array un tableau associatif d'un composant array('model'=>nom du modele,'manufacturer'=>id du fabricant,'type'=> id type);
     *** @throws \EsiaDbException
     **/
    public static function getImportComponent($dbconnection,$id=NULL)
    {
    	$query = "SELECT component_id, component_name,component_model,component_type_id,component_manufacturer_id,type_type,type_name,manufacturer_name
    			  FROM inventory_component
                  JOIN inventory_type ON type_id = component_type_id
                  JOIN inventory_manufacturer ON manufacturer_id = component_manufacturer_id";
    	$params = array();
    	if(!empty($id))
    	{
    		if(is_numeric($id))
    		{
    			$query .= " WHERE component_id=?";
    		}
    		else
    		{
    			$query .= " WHERE component_model=?";
    		}
    		$params[]=$id;
    	}
    	$query = $dbconnection->prepare($query);
    	$query = Model_esia::executeStatement($query,$params);
    	$arr = array();
    	while($row = $query->fetch(PDO::FETCH_OBJ))
    	{
    		$arr[$row->component_id]=array('name'=>$row->component_name,
                                            'model'=>$row->component_model,
                                            'manufacturer_id'=>$row->component_manufacturer_id,
                                            'manufacturer_name'=>$row->manufacturer_name,
                                            'type_id'=> $row->component_type_id,
                                            'type_name'=>$row->type_name,
                                            'type'=>$row->type_type
                                        ) ;
    	}
    	return $arr;
    }
    public static function getVendorByMac($dbconnection,$mac)
    {
        $mac = preg_replace('#^0x#','',$mac);
     	$query = "SELECT vendor_company
                  FROM  esia_mac_address_vendor
                  WHERE vendor_mac=trunc(?::macaddr)";
    	$query = $dbconnection->prepare($query);
    	$query = Model_esia::executeStatement($query,array($mac));
        $vendor ="";
        while($row = $query->fetch(PDO::FETCH_OBJ))
		{
            $vendor = $row->vendor_company;
            break;
        }
        return $vendor;
    }
	/**
	*** Récupère le nombre de règles dans chaque status (OK, OUTDATED) pour un type d'inventaire (software, hardware)
	*** @param PDO $dbconnection un objet PDO
	*** @param string (optionnel) $typeInventory permet de spécifier le type d'inventaire (software, hardware) pour lequel on doit récupérer les données (ALL pour tous)
	*** @return array Nombre de règles en OK, nombre de règles en OUTDATED
	*** @throws EsiaDbException
	**/
	public static function getCountRulesErrorByInventoryType($dbconnection, $typeInventory='ALL', $filterDevices=NULL)
    {
		$sql = "SELECT COUNT(CASE WHEN built_rule_match_worst_status = 'OK' THEN 1 END) AS built_rule_match_count_ok,
					   COUNT(CASE WHEN built_rule_match_worst_status = 'OUTDATED' THEN 1 END) AS built_rule_match_count_outdated
				FROM inventory_is_built";

		$params = array();
		$typeInventory = strtoupper($typeInventory);
		if($typeInventory != "ALL")
        {
            $sql .= "
                JOIN inventory_component ON inventory_is_built.built_hardware_id = inventory_component.component_id
                JOIN inventory_type ON inventory_component.component_type_id = inventory_type.type_id
                WHERE built_rule_match_worst_rule_id IS NOT NULL
                AND inventory_type.type_type = :inventory_type
            ";

			$params[':inventory_type'] = $typeInventory;
        }
		else
		{
			$sql .= " WHERE built_rule_match_worst_rule_id IS NOT NULL";
		}

        if(isset($filterDevices))
        {
            $in_sql = Model_esia::pdo_array_to_SQL('built_device_id', $filterDevices, $params);
            $sql .= " AND inventory_is_built.built_device_id IN ($in_sql)";
        }

		$query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query, $params);

		return $query->fetch(PDO::FETCH_ASSOC);
    }
    /**
	*** Récupère le nombre de status (OK, OUTDATED) par règle
	*** @param PDO $dbconnection un objet PDO
	*** @param integer OR array (optionnel) $rules permet de spécifier le ou les id de règles pour lequels on veut récupérer les status (NULL pour toutes les règles)
    *** @param array (optionnel) $filterDevices permet de spécifier un filtre sur des devices id
	*** @return array Par id de règle, nombre de status en OK, nombre de status en OUTDATED
	*** @throws EsiaDbException
	**/
	public static function getCountRulesErrorByRules($dbconnection, $rules=NULL, $filterDevices=NULL)
    {
		$sql = "SELECT built_rule_match_worst_rule_id,
                    COUNT(CASE WHEN built_rule_match_worst_status = 'OK' THEN 1 END) AS built_rule_match_count_ok,
				    COUNT(CASE WHEN built_rule_match_worst_status = 'OUTDATED' THEN 1 END) AS built_rule_match_count_outdated
			    FROM inventory_is_built";

		$params = array();

        if(is_numeric($rules))
        {
            $sql .= " WHERE built_rule_match_worst_rule_id = :rule_id";
            $params = array(':rule_id' => $rules);
        }
        elseif(is_array($rules))
        {
            $in_sql = Model_esia::pdo_array_to_SQL('rule_id', $filterDevices, $params);
            $sql .= " WHERE built_rule_match_worst_rule_id IN ($in_sql)";
        }
        else
        {
            // If rules ids are specified, else get all rules
            $sql .= " WHERE built_rule_match_worst_rule_id IS NOT NULL";
        }

        if(isset($filterDevices))
        {
            $in_sql = Model_esia::pdo_array_to_SQL('built_device_id', $filterDevices, $params);
            $sql .= " AND inventory_is_built.built_device_id IN ($in_sql)";
        }

        $sql .= "
            GROUP BY built_rule_match_worst_rule_id
            ORDER BY built_rule_match_worst_rule_id ASC
        ";

		$query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query, $params);

		$rulesList = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
            $rulesList[$row->built_rule_match_worst_rule_id] = array(
                'ok' => $row->built_rule_match_count_ok,
                'outdated' => $row->built_rule_match_count_outdated,
            );
		}

		return $rulesList;
    }
	/**
	*** Récupère les infos sur les règles en erreur (OUTDATED) pour un type d'inventaire (software, hardware)
	*** @param PDO $dbconnection un objet PDO
	*** @param string (optionnel) $typeInventory permet de spécifier le type d'inventaire (software, hardware) pour lequel on doit récupérer les données (ALL pour tous)
	*** @param integer (optionnel) $limit permet de spécifier un nombre limite de résultats
	*** @return array Liste de règles en erreur
	*** @throws EsiaDbException
	**/
	public static function getRulesInErrorByInventoryType($dbconnection, $typeInventory='ALL', $limit=5, $filterDevices=NULL)
    {
		$sql = "SELECT DISTINCT rule_id, rule_name, rule_version_target, component_name,
					(SELECT COUNT(*)
						FROM inventory_is_built
						WHERE built_rule_match_worst_rule_id=inventory_rules.rule_id
						AND built_rule_match_worst_rule_id IS NOT NULL
						AND built_rule_match_worst_status = 'OUTDATED'
					) built_rule_match_error_count
				FROM inventory_rules
				JOIN inventory_component ON inventory_rules.rule_component_id = inventory_component.component_id
				JOIN inventory_is_built ON inventory_component.component_id = inventory_is_built.built_hardware_id";

		$params = array();
		$typeInventory = strtoupper($typeInventory);
		if($typeInventory != "ALL")
        {
			$sql .= "
                JOIN inventory_type ON inventory_component.component_type_id = inventory_type.type_id
                WHERE inventory_type.type_type = :inventory_type
                AND inventory_is_built.built_rule_match_worst_rule_id IS NOT NULL
            ";

			$params[':inventory_type'] = $typeInventory;
        }
		else
		{
			$sql .= " WHERE inventory_is_built.built_rule_match_worst_rule_id IS NOT NULL";
		}

        if(isset($filterDevices))
        {
            $in_sql = Model_esia::pdo_array_to_SQL('built_device_id', $filterDevices, $params);
            $sql .= " AND inventory_is_built.built_device_id IN ($in_sql)";
        }

		$sql .= "
            ORDER BY built_rule_match_error_count DESC
            LIMIT :count_limit;
        ";

		$params[':count_limit'] = $limit;

		$query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query, $params);

        $rulesList = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
            if($row->built_rule_match_error_count > 0)
            {
                $rulesList[$row->rule_id] = array(
                    'name' => $row->rule_name,
                    'target_version' => $row->rule_version_target,
                    'component_name' => $row->component_name,
                    'match_count' => $row->built_rule_match_error_count,
                );
            }
		}

		return $rulesList;
    }
	/**
	*** Récupère le nombre de fois qu'un fabricant apparait pour un type d'inventaire (software, hardware)
	*** @param PDO $dbconnection un objet PDO
	*** @param string (optionnel) $typeInventory permet de spécifier le type d'inventaire (software, hardware) pour lequel on doit récupérer les données (ALL pour tous)
	*** @param integer (optionnel) $limit permet de spécifier un nombre limite de résultats
	*** @return array Liste de fabricants
	*** @throws EsiaDbException
	**/
	public static function getCountManufacturerByInventoryType($dbconnection, $typeInventory='ALL', $limit=5, $node=NULL, $filterDevices=NULL)
    {
		$sql = "SELECT manufacturer_name, COUNT(*) AS manufacturer_count
			 	FROM inventory_manufacturer
				JOIN inventory_component ON inventory_manufacturer.manufacturer_id = inventory_component.component_manufacturer_id
				JOIN inventory_is_built ON inventory_component.component_id = inventory_is_built.built_hardware_id";

		$params = array();
		$typeInventory = strtoupper($typeInventory);
		if($typeInventory != "ALL")
        {
			$sql .= " JOIN inventory_type ON inventory_component.component_type_id = inventory_type.type_id";
            $sql .= " WHERE manufacturer_name <> 'unknown' AND inventory_type.type_type = :inventory_type";

			$params[':inventory_type'] = $typeInventory;
        }
		else
		{
			$sql .= " WHERE manufacturer_name <> 'unknown'";
		}
        if($node != NULL)
        {
            $sql .= " AND inventory_is_built.built_device_id = :node";
            $params[':node'] = $node;
        }

        if(isset($filterDevices))
        {
            $in_sql = Model_esia::pdo_array_to_SQL('built_device_id', $filterDevices, $params);
            $sql .= " AND inventory_is_built.built_device_id IN ($in_sql)";
        }

		$sql .= " GROUP BY manufacturer_name";
		$sql .= " ORDER BY manufacturer_count DESC";
		$sql .= " LIMIT :count_limit";

		$params[':count_limit'] = $limit;
		$query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query, $params);

        $manufacturerList = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$manufacturerList[$row->manufacturer_name] = array(
				'name' => $row->manufacturer_name,
				'count' => $row->manufacturer_count,
			);
		}
		return $manufacturerList;
    }
    public static function getReportRules($dbconnection,$scope,$follow,$array_id)
    {
        $params = null;
        $where_exist = false;
        $query = "SELECT rule_name,device_name,built_name,built_rule_match_worst_status,built_version
            FROM inventory_manufacturer
            INNER JOIN inventory_component on inventory_manufacturer.manufacturer_id = inventory_component.component_manufacturer_id
            INNER JOIN inventory_is_built ON inventory_component.component_id = inventory_is_built.built_hardware_id
            INNER JOIN esia_device ON inventory_is_built.built_device_id = esia_device.device_id
            INNER JOIN inventory_type ON inventory_type.type_id = inventory_component.component_type_id
			INNER JOIN inventory_rules ON inventory_rules.rule_id = inventory_is_built.built_rule_match_worst_rule_id";

        if($scope == "NODE" || $scope == "GROUPNODE")
        {
            $where_exist = true;
            $query .= " WHERE ";
            $columns = " device_id ";
            if(!is_array($array_id))
            {
                $query .= " $columns = :dataid";
                $params[':dataid'] = $array_id;
            }
            else
            {
                $i = 0;
                $sql_prepare_arr='';
                foreach($array_id as $id)
                {
                    $params[':dataid'.$i]=$id;
                    $sql_prepare_arr .= ':dataid'.$i.',';
                    $i++;
                }
                $sql_prepare_arr= rtrim($sql_prepare_arr,',');
                $query .= " $columns IN ($sql_prepare_arr) ";
            }

        }
        if($follow != "ALL")
        {
            if($where_exist)
            {
                $query .= " AND inventory_is_built.built_rule_match_worst_status = :follow ";
            }
            else
            {
                $query .= " WHERE inventory_is_built.built_rule_match_worst_status = :follow ";
            }
            $params[':follow'] = $follow;
        }
        $query = $dbconnection->prepare($query);
        $query = Model_esia::executeStatement($query, $params);

        $rules = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$rules[$row->device_name] = array(
				'rule_name' => $row->rule_name,
				'device_name' => $row->device_name,
                'built_name' => $row->built_name,
                'built_rule_match_worst_status'=> $row->built_rule_match_worst_status,
                'built_version'=>$row->built_version,
			);
		}

		return $rules;
    }
    public static function getInventoryTreeView($dbconnection, $userid, $groupid=NULL)
	{
        $group_list = array();
        // Get group children to build treeview on group click
        $sql = "
        WITH accessgroup AS (
            SELECT device_group_id,esia_device_group.device_group_name,device_group_parent
            FROM esia_device_group
            LEFT JOIN esia_device_group_in ON device_group_id = group_in_device_group_id
            LEFT JOIN esia_device ON device_id = group_in_device_id
            WHERE device_id IN (SELECT * FROM getUserAccessNode(:userid))
            OR (
                device_id IS NULL
                AND device_group_id IN (SELECT device_group_parent FROM esia_device_group)
            )
            GROUP BY device_group_id,device_group_parent
        )
        SELECT *
        FROM accessgroup
		";

        $params = array(':userid' => $userid);

        if(empty($groupid))
        {
            $sql .= "
                WHERE device_group_parent IS null
                OR device_group_parent IN (SELECT device_group_id FROM accessgroup WHERE device_group_parent IS null)
            ";
        }
        else
        {
            $sql .= "
                WHERE device_group_parent = :groupid
                OR device_group_parent IN (SELECT device_group_id FROM accessgroup WHERE device_group_parent = :groupid)
            ";

            $params[':groupid'] = $groupid;
        }

        $sql .= " ORDER BY upper(device_group_name) ASC;";

        $query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query, $params);

        $hasChildren = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
            if(empty($row->device_group_parent) || $groupid == $row->device_group_parent)
            {
                // Direct children of the requested group
                $group_list[$row->device_group_id] = array(
                    'id' => $row->device_group_id,
                    'name' => $row->device_group_name,
                    'parent' => $row->device_group_parent,
                    'hasChildren' => FALSE,
                );
            }
            else
            {
                // Children of second level. Save parent ID to set has children to the parent (key is set to prevent duplication and keep this ids list unique)
                $hasChildren[$row->device_group_parent] = $row->device_group_parent;
            }
		}
        // When the group_list is fully created set hasChildren true for each group present in hasChildren array.
        foreach($hasChildren as $id)
        {
            $group_list[$id]['hasChildren'] = TRUE;
        }

        // Remove keys as device_group_id to re-index the array keeping the sql order result and prevent the array to be resort by device_group_id. device_group_id key was only neccessary to find the right group to set hasChildren
        $group_list = array_values($group_list);

		return $group_list;
    }
    public static function getGroupById($dbconnection, $id)
    {
        if(empty($id))
        {
            throw new EsiaDbException('There is no group ID');
        }

        $sql = "SELECT * FROM esia_device_group";

        $params = array();

        if(is_array($id))
        {
            $in_sql = Model_esia::pdo_array_to_SQL('device_group_id', $id, $params);
            $sql .= " WHERE device_group_id IN ($in_sql) ORDER BY device_group_name;";
        }
        elseif(is_numeric($id))
        {
            $sql .= " WHERE device_group_id = :id;";
            $params = array(':id' => $id);
        }

        $query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query, $params);

        return $query->fetchAll(PDO::FETCH_ASSOC);
    }
    public static function getInventoryDevicesIdByGroup($dbconnection, $userid, $groupid)
    {
        $sql = "
            SELECT device_id
            FROM esia_device
            LEFT JOIN esia_device_group_in ON group_in_device_id = device_id
            LEFT JOIN esia_device_group ON device_group_id = group_in_device_group_id
            WHERE device_id IN (SELECT * FROM getUserAccessNode(:userid))
            AND device_group_id IN (SELECT group_id from getallchilddevicegroup(:groupid))
        ";

        $params = array(
            ':userid' => $userid,
            ':groupid' => $groupid,
        );

        $query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query, $params);

        return $query->fetchAll(PDO::FETCH_COLUMN);
    }
    public static function getUserAccessDevicesNumber($dbconnection, $userid)
    {
        $query = $dbconnection->prepare("SELECT COUNT(*) FROM getUserAccessNode(:userid)");
        $query = Model_esia::executeStatement($query, array(':userid' => $userid));

        return $query->fetch(PDO::FETCH_COLUMN);
    }
    public static function getInventoryBuiltTypesCount($dbconnection, $userid, $filterDevices=NULL)
    {
        $sql = "
            SELECT
                SUM(CASE WHEN inventory_type.type_type = 'HARDWARE' THEN 1 END) AS nbr_hardwares,
                SUM(CASE WHEN inventory_type.type_type = 'SOFTWARE' THEN 1 END) AS nbr_softwares
            FROM inventory_is_built
            LEFT JOIN inventory_component ON inventory_component.component_id = inventory_is_built.built_hardware_id
            LEFT JOIN inventory_type ON inventory_type.type_id = inventory_component.component_type_id
            WHERE inventory_is_built.built_device_id IN (SELECT * FROM getUserAccessNode(:userid))
        ";

        $params = array(':userid' => $userid);

        if(isset($filterDevices))
        {
            $in_sql = Model_esia::pdo_array_to_SQL('built_device_id', $filterDevices, $params);
            $sql .= " AND inventory_is_built.built_device_id IN ($in_sql)";
        }

        $query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query, $params);

        $result = $query->fetch(PDO::FETCH_ASSOC);

        // If not match SUM return null so set column to 0
        if(!isset($result['nbr_hardwares']))
        {
            $result['nbr_hardwares'] = 0;
        }
        if(!isset($result['nbr_softwares']))
        {
            $result['nbr_softwares'] = 0;
        }

        return $result;
    }
    public static function getScannedRuleReport($dbconnection, $rulesId=NULL, $matchStatus=NULL)
    {
        $sql = "
            SELECT inventory_is_built.built_rule_match_worst_rule_id,inventory_is_built.built_name,inventory_is_built.built_description,inventory_is_built.built_version,inventory_is_built.built_rule_match_worst_status,
                inventory_component.component_name,inventory_component.component_description,inventory_component.component_model,
                inventory_type.type_name,inventory_type.type_type,
                inventory_manufacturer.manufacturer_name,
                esia_device.device_name,esia_device_group.device_group_name
            FROM inventory_is_built
            LEFT JOIN inventory_component on inventory_is_built.built_hardware_id = inventory_component.component_id
            LEFT JOIN inventory_type on inventory_component.component_type_id = inventory_type.type_id
            LEFT JOIN inventory_manufacturer on inventory_component.component_manufacturer_id = inventory_manufacturer.manufacturer_id
            LEFT JOIN esia_device ON inventory_is_built.built_device_id = esia_device.device_id
            LEFT JOIN esia_device_group_in ON esia_device_group_in.group_in_device_id = esia_device.device_id
            LEFT JOIN esia_device_group ON esia_device_group.device_group_id = esia_device_group_in.group_in_device_group_id
        ";

        $params = array();

        if(isset($rulesId))
        {
            if(is_array($rulesId))
            {
                $in_sql = Model_esia::pdo_array_to_SQL('built_rule_match_worst_rule_id', $rulesId, $params);
                $sql .= " WHERE inventory_is_built.built_rule_match_worst_rule_id IN ($in_sql)";
            }
            elseif(is_numeric($rulesId) && $rulesId > 0)
            {
                $sql .= " WHERE inventory_is_built.built_rule_match_worst_rule_id = :rule_id";
                $params[':rule_id'] = $rulesId;
            }
        }
        else
        {
            // All scanned rules (built_rule_match_worst_rule_id is set)
            $sql .= " WHERE inventory_is_built.built_rule_match_worst_rule_id IS NOT null";
        }

        if(isset($matchStatus))
        {
            // Filter on rule match status
            $sql .= " AND inventory_is_built.built_rule_match_worst_status = :match_status";
            $params[':match_status'] = $matchStatus;
        }

        $sql .= " ORDER BY inventory_is_built.built_rule_match_worst_rule_id,inventory_is_built.built_rule_match_worst_status,esia_device.device_name";

        $query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query, $params);

        return $query->fetchAll(PDO::FETCH_ASSOC);
    }
    public static function getGroupedTypeComponents($dbconnection)
    {
        $types = array();
        $sql = "SELECT type_name FROM inventory_type ORDER BY type_type ASC";
        $query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query);

        while($row = $query->fetch(PDO::FETCH_OBJ))
        {
            $types[$row->type_name] = array(
                'type_name' => $row->type_name
            );
        }
        return $types;
    }
    public static function getGroupedComponents($dbconnection,$type)
    {
        $components = array();

        $sql = "
            SELECT DISTINCT ON (component_name)component_name, component_id,type_name
            FROM inventory_component
            LEFT JOIN inventory_type ON inventory_component.component_type_id = inventory_type.type_id
            WHERE inventory_type.type_name = :type
            ORDER BY component_name ASC
        ";

        $query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query, array(':type' => $type));

        while($row = $query->fetch(PDO::FETCH_OBJ))
        {
            $components[$row->type_name][] = array(
                'id' => $row->component_id,
                'name' => $row->component_name,
            );
        }

        return $components;
    }
    public static function getTypeByComponentId($dbconnection,$component_id)
    {
        $query = "
            SELECT type_name
            FROM inventory_type
            INNER JOIN inventory_component ON inventory_component.component_type_id = inventory_type.type_id
            WHERE component_id = :component_id
        ";

        $query = $dbconnection->prepare($query);
        $query = Model_esia::executeStatement($query, array(':component_id' => $component_id));
        return $query->fetch();
    }

    /**
	*** Get all node groups on which an user has access rights on childs nodes. Get even groups with no nodes children but only if they have children groups with nodes. Return group name by hierachy parent>group.
	*** @param PDO $dbconnection PDO object
	*** @param integer $userId user id to check groups access
	*** @return array Group list
	*** @throws EsiaDbException
	**/
    public static function getAvaillableNodeGroupsByUser($dbconnection, $userId)
	{
        $sql = "
            WITH accessgroup AS (
                SELECT device_group_id,esia_device_group.device_group_name,device_group_parent
                FROM esia_device_group
                LEFT JOIN esia_device_group_in ON device_group_id = group_in_device_group_id
                LEFT JOIN esia_device ON device_id = group_in_device_id
                WHERE device_id IN (SELECT * FROM getUserAccessNode(:userid))
                OR (
                    device_id IS NULL
                    AND device_group_id IN (SELECT device_group_parent FROM esia_device_group)
                )
                GROUP BY device_group_id,device_group_parent
            )
            SELECT *
            FROM accessgroup
            ORDER BY device_group_parent DESC, device_group_name ASC;
        ";

        $query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query, array(':userid' => $userId));

        $groupList = array();
        $indexedGroups = array();
        while($row = $query->fetch(PDO::FETCH_OBJ))
        {
            $indexedGroups[$row->device_group_id] = array(
                'id' => $row->device_group_id,
                'name' => $row->device_group_name,
                'parent' => $row->device_group_parent,
            );
        }
        foreach($indexedGroups as $group)
        {
            $grpItem = array(
                'id' => $group['id'],
                'name' => $group['name'],
            );

            $parent = $group;
            while($parent['parent'] != NULL)
            {
                $parent = $indexedGroups[$parent['parent']];
                $grpItem['name'] = $parent['name'] . '>' . $grpItem['name'];
            }
            $groupList[] = $grpItem;
        }

		return $groupList;
    }

    /**
    *** Get all nodes which have SNMP plugins with all SNMP plugins in UNKNOWN state.
    *** @param PDO $dbconnection PDO object
    *** @param NULL|bool $is_remote NULL for all, TRUE for nodes linked to Unity and FALSE for direct nodes
    *** @return array list into format $arr[$node_id] = $nodeid
    *** @throws EsiaDbException
    **/
    public static function getAllNodesWithSNMPServicesUnknown($dbconnection,$is_remote=NULL)
    {
        if( $is_remote === TRUE )
        {
            $filter_plugin = 'GESA_SNMP';
        }
        elseif( $is_remote === FALSE )
        {
            $filter_plugin = 'CHECK_SNMP';
        }
        else
        {
            $filter_plugin = 'SNMP';
        }
        $sql = "
            SELECT service_device_id
            FROM (
                SELECT  service_device_id,
                    COUNT(service_id ) FILTER (WHERE service_previous_state = 3) as unknown,
                    COUNT(service_id ) FILTER (WHERE service_previous_state <> 3) as other
                FROM esia_services
                JOIN esia_plugins ON service_plugin_id = plugin_id
                WHERE plugin_name LIKE '$filter_plugin%'
                GROUP BY service_device_id) AS T1
            WHERE T1.unknown > 0 AND T1.other=0;
        ";

        $query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query);
        $ret = array();
        while($row = $query->fetch(PDO::FETCH_OBJ))
        {
            $ret[$row->service_device_id] = $row->service_device_id;
        }
        return $ret;
    }
     /**
	*** Get all microsoft KBs detected by the inventory scan where the CVE list is not set yet.
	*** @param PDO $dbconnection un objet PDO
    *** @return array list of KB components where CVE list is not found
	*** @throws EsiaDbException
	**/
	public static function getComponentKbWithEmptyCve($dbconnection)
	{
        $sql = "SELECT component_id, component_name, component_json
				FROM inventory_component
				WHERE component_type_id = (SELECT type_id FROM inventory_type WHERE type_name = 'KB')
                AND component_json->'CVE' IS NULL;";

        $query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, array());

        $kbList = array();
        while($row = $query->fetch(PDO::FETCH_OBJ))
        {
            $kbList[$row->component_id] = array(
                'name' => $row->component_name,
                'json' => $row->component_json,
            );
        }

        return $kbList;
    }
    /**
	*** Update inventory_component json
	*** @param PDO $dbconnection un objet PDO
	*** @param integer|string $component component id or component name of the KB to update
    *** @param array $jsonArr full component json with the added CVE list of the KB
	*** @throws EsiaDbException
	**/
	public static function updateComponentJSON($dbconnection, $component, $jsonArr)
	{
        $sql = "UPDATE inventory_component
                SET component_json = :comp_json";

        if(empty($component))
        {
            throw new EsiaDbException('Missing component ID or Name');
        }
        elseif(is_numeric($component))
        {
            $sql .= " WHERE component_id = :component;";
        }
        elseif(is_string($component))
        {
            $sql .= " WHERE component_name = :component;";
        }
        else
        {
            throw new EsiaDbException('Wrong component identifier type (integer or string)');
        }

        $params = array(
            ':component' => $component,
            ':comp_json' => json_encode($jsonArr),
        );

        $query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);
    }
}
?>
