#!/usr/bin/php
<?php
$web_path = "/usr/local/esia/public_html/";
$_SERVER['DOCUMENT_ROOT'] = $web_path;

// Inclusion des fichiers
include_once ($web_path."esia.conf.php");
include_once ($web_path."Model_esia.php");
include_once($web_path."plugin/inventory/Model_inventory.php");

try
{
	$dbconnection = new PDO('pgsql:host='.DB_HOST.';port='.DB_PORT.';dbname='.DB_NAME, DB_USERNAME, DB_PWD);
	$dbconnection->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
}
catch(Exception $e)
{
	echo 'PDO SQL ERROR: '.$e->getMessage() . "\n";
	echo 'N° : '.$e->getCode() . "\n";
}

try
{
	$rules = Model_inventory::getListRules($dbconnection,NULL,TRUE);
}
catch(Exception $e)
{
	echo "GLOBAL ERROR: " . $e->getMessage() . "\n";
	exit(0);
}

// Identification des règles a traiter
foreach($rules as $rule)
{
	try
	{
		echo "CHECK RULE ".$rule['name'].": version ".$rule['target_version']." operator ". $rule['operator_target']."\n";
        $version = $rule['target_version'];
        $operator = $rule['operator_target'];
        $nodes = array();
        if(!empty($rule['target']['devices']))
        {
            $nodes = Model_inventory::getDeviceComponent($dbconnection, $rule['target']['devices'], $rule['component_name']);
        }
        if(!empty($rule['target']['groups']))
        {
            $group = Model_inventory::getDeviceComponent($dbconnection, $rule['target']['groups'], "GROUP", $rule['component_name']);
            $nodes = array_merge($nodes, $group);
        }
        $matchs = array();
        foreach($nodes as $node)
        {
            $id = $node['built_id'];
            echo "Device ".$node['device']." install version is  $version $operator ".$node['version']."\n";
            if(cmpVersion($node['version'],$version,$operator))
            {
                $matchs[$id]='OK';
            }
            else
            {
                $matchs[$id]='OUTDATED';
            }
        }
        Model_inventory::updateMatchToRule($dbconnection,$matchs,$rule['rule_id']);
	}
	catch(EsiaDbException $e)
	{
		echo $rule['rule_name'] . " DB ERROR: " . $e->getMessage() . "\n";
	}
	catch(Exception $e)
	{
		echo $rule['rule_name'] . " ERROR: " . $e->getMessage() . "\n";
	}
}
echo "END check RULES\n";
exit(0);
function cmpVersion($ver1,$ver2,$operator)
{
    $return =  FALSE;
    if($operator == '=')
    {
        if($ver1 == $ver2)
        {
            $return = TRUE;
        }
    }
    elseif(preg_match('#[0-9\.]*#',$ver1))
    {
        //si la version est une série de chiffre séparée par des points
            $arr_ver1 = explode('.',$ver1);
            $arr_ver2 = explode('.',$ver2);        
            $max1 = count($arr_ver1);
            $max2 = count($arr_ver2);
            if($max1 > $max2)
            {
                $delta = $max1 - $max2;
                for($i=0;$i < $delta;$i++)
                {
                    $arr_ver2[]=0;
                }
            }
            elseif($max1 < $max2)
            {
                $delta = $max2 - $max1;
                for($i=0;$i < $delta;$i++)
                {
                    $arr_ver1[]=0;
                }
            }
            $i = 0;
            $size = count($arr_ver1);
            $flag_end = FALSE;
            while($flag_end == FALSE)
            {

                $elem1 = $arr_ver1[$i];
                $elem2 = $arr_ver2[$i];
                //echo "\t$elem1 $operator $elem2\n";
                if($operator == '>')
                {
                    if($elem1 > $elem2)
                    {
                        $return = TRUE; 
                        $flag_end = TRUE;
                    }
                    else
                    {
                        $return = FALSE; 
                        $flag_end = TRUE;
                    } 
                }
                elseif($operator == '>=')
                {
                    if($elem1 >= $elem2)
                    {
                        $return = TRUE; 
                        $flag_end = TRUE;
                    }
                    else
                    {
                        $return = FALSE; 
                        $flag_end = TRUE;
                    } 
                }
                elseif($operator == '<=')
                {
                    if($elem1 <= $elem2)
                    {
                        $return = TRUE; 
                        $flag_end = TRUE;
                    } 
                    else
                    {
                        $return = FALSE; 
                        $flag_end = TRUE;
                    } 
                }
                elseif($operator == '<')
                {
                    if($elem1 < $elem2)
                    {
                        $return = TRUE; 
                        $flag_end = TRUE;
                    } 
                    else
                    {
                        $return = FALSE; 
                        $flag_end = TRUE;
                    } 
                }
                $i++;
                if($i > $size)
                {
                    $flag_end = TRUE;
                }
            }
    }
    else
    {
        $cmp = strcmp($ver1, $ver2);
        if($operator == '>' && $cmp == 1)
        {
            $return = TRUE; 
        }
        elseif($operator == '>=' && ($cmp == 1 || $cmp == 0))
        {
            $return = TRUE; 
        }
        elseif($operator == '<' && $cmp == -1)
        {
            $return = TRUE; 
        }
        elseif($operator == '<=' && ($cmp == -1 || $cmp == 0))
        {
            $return = TRUE; 
        }
    }
    return $return;
}
?>
