/* control_inventory.js */

// Configure table sorter
function tableSorter(idTable,idPager)
{
	Utils.createTable(idTable);
	$(idTable).tablesorterPager({
		container: $( idPager ),
		output: '{startRow} to {endRow} ({filteredRows})',
		size: 50,
	});
	$(idTable).data("tablesorter").widgets.push('print');
	$(idTable).data("tablesorter").widgets.push('columnSelector');
	$(idTable).data("tablesorter").widgetOptions['columnSelector_container']=$(idTable+'_columnSelector');
	$(idTable+'_print').on('click', function(){
		$(idTable).trigger('printTable');
	});
}

// Annimation to fade an user message
function fadeUserMessage(item)
{
	item.delay(4000).fadeOut(500);
}

// Configure jquery ui tabs
function setTabs(idTabs)
{
	$(idTabs).tabs({
		heightStyle: "content",
		beforeLoad: function( event, ui ) {
			$('#esiaOverlay').show();
			$('#loadingGif').show();

			ui.jqXHR.fail(function() {
				ui.panel.html(
					"Couldn't load this tab. We'll try to fix this as soon as possible. " +
					"If this wouldn't be a demo."
				);
				$('#esiaOverlay').hide();
				$('#loadingGif').hide();
			});
		},
		load: function (event, ui){
			$('#esiaOverlay').hide();
			$('#loadingGif').hide();
			if(ui.panel.find("#page_login_detect").length > 0) location.reload();
		},
	});
}

// Display error or success messages
function displayUserMessage(message, isError=true, idHtml="messages")
{
	var container = $("<div>", {"class": "messageUtilisateurs"});

	if(isError)
	{
		container.addClass("EventCritical");
	}
	else
	{
		container.addClass("EventOK");
	}

	if($.isArray(message))
	{
		$.each(message, function()
		{
			container.append("<p>" + this + "</p>");
		});
	}
	else
	{
		container.append("<p>" + message + "</p>");
	}

	// Display new message during 5s
	$("#" + idHtml).append(container);
	fadeUserMessage(container);
}

// Perform a Jquery ajax request with the same behaviour as a common Promise to use it with (async/await statements and error handeling with try catch)
// Usage:
// $.ajax({type: "POST",url: url, ... }).done(data => {return data;});
// ajaxPromise({type: "POST",url: url, ... }).then(data => {return data;});
function ajaxPromise(settings)
{
	return new Promise((resolve, reject) => {
		$.ajax(settings)
		.done(data => resolve(data))
		.fail((jqXHR, textStatus, errorThrown) => reject(errorThrown));
	});
}

// Perform an ajax request with response in json usable in try catch async/await statement. Error handeling is preconfigured for php error trow
function asyncSaveData(settings)
{
	return ajaxPromise({
		type: "POST",
		...settings,
		dataType: "json",
	})
	.then(data => {
		if(data.errors != undefined)
		{
			// Error returned by php
			throw new Error(data.errors);
		} else {
			return data;
		}
	});
}

// Return all inputs, select, ... of an html form as a data object (like used in $.ajax data: {}).
function getFormValues(idHtml)
{
	return $('#' + idHtml).serializeArray().reduce(function(o, input)
	{
		if(input.name.slice(-2) === '[]')
		{
			// If input name ends by [] its val should be an array
			// Remove [] from the name
			input.name = input.name.slice(0, -2);
			if(!o[input.name])
			{
				// If the array key doesn't exist create it and initialize the first array value
				o[input.name] = [input.value];
			}
			else
			{
				// The array key already exist push the new value into the array
				o[input.name].push(input.value);
			}
		}
		else
		{
			o[input.name] = input.value;
		}

		return o;
	}, {});
}

// Update scan rules config by ajax request
async function updateScanRulesConfig()
{
	$("#invScanRulesConfig").on('submit', async e => {
		e.preventDefault();

		try
		{
			const formData = getFormValues('invScanRulesConfig');
			
			const success = await asyncSaveData({
				url: glob_plugin_url + 'display_inventory.php',
				data: {
					action: 'updateScanRulesConfig',
					...formData,
				},
			});

			// If no error set success message
			displayUserMessage(success, false);
		}
		catch(error)
		{
			displayUserMessage(error, true);
		}
	});
}

// Set a rules scan now request for rules scan
async function execScanRules()
{
	$("#btnExecNowRules").on("click", async () => {
		try
		{
			const success = await asyncSaveData({
				url: glob_plugin_url + 'display_inventory.php',
				data: {
					action: 'execScanRulesNow',
				},
			});

			// If no error set success message
			displayUserMessage(success, false);
		}
		catch(error)
		{
			displayUserMessage(error, true);
		}
	});
}

// Create the donut for elements rules match
function setRulesMatchDonut(id, data, size=180)
{
	let isEmpty = false;
	
	if(data[0][1] == 0 && data[1][1] == 0) {
		data[0][1] = 1;
		isEmpty = true;
	}
	
	bb.generate({
		bindto: '#' + id,
		data: {
			type: 'donut',
			columns: data,
		},
		color: {
			pattern: ['#83BC00', '#DA002E'],
		},
		legend: {
			show: false,
		},
		size: {
			width: size,
			height: size,
		},
		tooltip: {
			show: !isEmpty,
			format: {
				value: value => d3.format(',')(value),
			},
		},
	});
}

// Create the bar chart for manufacturer components
function setComponentManBarChart(id, data, size=180)
{
	bb.generate({
		bindto: '#' + id,
		data: {
			type: 'bar',
			columns: data.values,
		},
		size: {
			height: size,
		},
		bar: {
			width: {
				ratio: 0.8,
			},
		},
		axis: {
			rotated: true,
			x: {
				type: 'category',
				categories: [''],
			},
		},
		legend: {
			show: true,
			position: 'right',
		},
		grid: {
			y: {
				show: true,
			},
		},
		tooltip: {
			format: {
				title: () => data.label,
			},
		},
	});
}

// Create donut for top 10 elements (manufacturer, os)
function setInvTopDonut(id, data, height=300, title='')
{
	let isEmpty = false;
	if(typeof data === 'string')
	{
		data = [[data, 1]];
		isEmpty = true;
	}
	
	let settings = {
		bindto: '#' + id,
		data: {
			type: 'donut',
			columns: data,
		},
		donut: {
			title: title,
		},
		legend: {
			show: true,
			position: 'right',
		},
		size: {
			height: height + 'px',
		},
		tooltip: {
			show: !isEmpty,
			format: {
				value: value => d3.format(',')(value),
			},
		},
	};

	if(isEmpty)
	{
		settings.color = { pattern: ['#575759'] };
	}

	bb.generate(settings);
}
